@@ -1,8 +1,10 @@
-import json
 import pickle
 import sys
 import io
 import os
+import json
+import orjson
+import uuid
 import logging
 import re  # NOQA
 import builtins  # NOQA
@@ -20,11 +22,14 @@ except ImportError:  # pragma: no cover.
     toml = None  # pragma: no cover.
 try:
     import clevercsv
+    csv = None
 except ImportError:  # pragma: no cover.
+    import csv
     clevercsv = None  # pragma: no cover.
 from copy import deepcopy
+from functools import partial
 from collections.abc import Mapping
-from deepdiff.helper import (strings, json_convertor_default, get_type, TEXT_VIEW)
+from deepdiff.helper import (strings, get_type, TEXT_VIEW)
 from deepdiff.model import DeltaResult
 
 logger = logging.getLogger(__name__)
@@ -69,6 +74,7 @@ SAFE_TO_IMPORT = {
     'datetime.time',
     'datetime.timedelta',
     'decimal.Decimal',
+    'uuid.UUID',
     'ordered_set.OrderedSet',
     'collections.namedtuple',
     'collections.OrderedDict',
@@ -76,6 +82,34 @@ SAFE_TO_IMPORT = {
 }
 
 
+TYPE_STR_TO_TYPE = {
+    'range': range,
+    'complex': complex,
+    'set': set,
+    'frozenset': frozenset,
+    'slice': slice,
+    'str': str,
+    'bytes': bytes,
+    'list': list,
+    'tuple': tuple,
+    'int': int,
+    'float': float,
+    'dict': dict,
+    'bool': bool,
+    'bin': bin,
+    'None': None,
+    'NoneType': None,
+    'datetime': datetime.datetime,
+    'time': datetime.time,
+    'timedelta': datetime.timedelta,
+    'Decimal': decimal.Decimal,
+    'OrderedSet': ordered_set.OrderedSet,
+    'namedtuple': collections.namedtuple,
+    'OrderedDict': collections.OrderedDict,
+    'Pattern': re.Pattern,    
+}
+
+
 class ModuleNotFoundError(ImportError):
     """
     Raised when the module is not found in sys.modules
@@ -148,7 +182,7 @@ class SerializationMixin:
             '{"type_changes": {"root": {"old_type": "A", "new_type": "B", "old_value": "obj A", "new_value": "obj B"}}}'
         """
         dic = self.to_dict(view_override=TEXT_VIEW)
-        return json.dumps(dic, default=json_convertor_default(default_mapping=default_mapping), **kwargs)
+        return json_dumps(dic, default_mapping=default_mapping, **kwargs)
 
     def to_dict(self, view_override=None):
         """
@@ -377,7 +411,7 @@ def load_path_content(path, file_type=None):
         file_type = path.split('.')[-1]
     if file_type == 'json':
         with open(path, 'r') as the_file:
-            content = json.load(the_file)
+            content = json_loads(the_file.read())
     elif file_type in {'yaml', 'yml'}:
         if yaml is None:  # pragma: no cover.
             raise ImportError('Pyyaml needs to be installed.')  # pragma: no cover.
@@ -393,9 +427,11 @@ def load_path_content(path, file_type=None):
             content = the_file.read()
             content = pickle_load(content)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv is None:  # pragma: no cover.
-            raise ImportError('CleverCSV needs to be installed.')  # pragma: no cover.
-        content = clevercsv.read_dicts(path)
+        if clevercsv:  # pragma: no cover.
+            content = clevercsv.read_dicts(path)
+        else:
+            with open(path, 'r') as the_file:
+                content = list(csv.DictReader(the_file))
         logger.info(f"NOTE: CSV content was empty in {path}")
 
         # Everything in csv is string but we try to automatically convert any numbers we find
@@ -439,7 +475,8 @@ def save_content_to_path(content, path, file_type=None, keep_backup=True):
 def _save_content(content, path, file_type, keep_backup=True):
     if file_type == 'json':
         with open(path, 'w') as the_file:
-            content = json.dump(content, the_file)
+            content = json_dumps(content)
+            the_file.write(content)
     elif file_type in {'yaml', 'yml'}:
         if yaml is None:  # pragma: no cover.
             raise ImportError('Pyyaml needs to be installed.')  # pragma: no cover.
@@ -454,14 +491,80 @@ def _save_content(content, path, file_type, keep_backup=True):
         with open(path, 'wb') as the_file:
             content = pickle_dump(content, file_obj=the_file)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv is None:  # pragma: no cover.
-            raise ImportError('CleverCSV needs to be installed.')  # pragma: no cover.
+        if clevercsv:  # pragma: no cover.
+            dict_writer = clevercsv.DictWriter
+        else:
+            dict_writer = csv.DictWriter
         with open(path, 'w', newline='') as csvfile:
             fieldnames = list(content[0].keys())
-            writer = clevercsv.DictWriter(csvfile, fieldnames=fieldnames)
+            writer = dict_writer(csvfile, fieldnames=fieldnames)
             writer.writeheader()
             writer.writerows(content)
     else:
         raise UnsupportedFormatErr('Only json, yaml, toml, csv, tsv and pickle are supported.\n'
                                    f' The {file_type} extension is not known.')
     return content
+
+
+def _serialize_decimal(value):
+    if value.as_tuple().exponent == 0:
+        return int(value)
+    else:
+        return float(value)
+
+
+JSON_CONVERTOR = {
+    decimal.Decimal: _serialize_decimal,
+    ordered_set.OrderedSet: list,
+    type: lambda x: x.__name__,
+    bytes: lambda x: x.decode('utf-8'),
+    datetime.datetime: lambda x: x.isoformat(),
+    uuid.UUID: lambda x: str(x),
+}
+
+
+def json_convertor_default(default_mapping=None):
+    if default_mapping:
+        _convertor_mapping = JSON_CONVERTOR.copy()
+        _convertor_mapping.update(default_mapping)
+    else:
+        _convertor_mapping = JSON_CONVERTOR
+
+    def _convertor(obj):
+        for original_type, convert_to in _convertor_mapping.items():
+            if isinstance(obj, original_type):
+                return convert_to(obj)
+        raise TypeError('We do not know how to convert {} of type {} for json serialization. Please pass the default_mapping parameter with proper mapping of the object to a basic python type.'.format(obj, type(obj)))
+
+    return _convertor
+
+
+class JSONDecoder(json.JSONDecoder):
+
+    def __init__(self, *args, **kwargs):
+        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
+
+    def object_hook(self, obj):
+        if 'old_type' in obj and 'new_type' in obj:
+            for type_key in ('old_type', 'new_type'):
+                type_str = obj[type_key]
+                obj[type_key] = TYPE_STR_TO_TYPE.get(type_str, type_str)
+
+        return obj
+
+
+def json_dumps(item, default_mapping=None, **kwargs):
+    """
+    Dump json with extra details that are not normally json serializable
+
+    Note: I tried to replace json with orjson for its speed. It does work
+    but the output it makes is a byte object and Postgres couldn't directly use it without
+    encoding to str. So I switched back to json.
+    """
+    return orjson.dumps(
+        item,
+        default=json_convertor_default(default_mapping=default_mapping),
+        **kwargs).decode(encoding='utf-8')
+
+
+json_loads = partial(json.loads, cls=JSONDecoder)
